@@ -25,6 +25,7 @@
 #  undef __XSI_VISIBLE /* redefined in config.h */
 #endif
 #include "config.h"
+#include <stddef.h>
 #include <stdint.h>
 #include <inttypes.h>
 #include "dwg.h"
@@ -37,49 +38,72 @@
    GCC46_DIAG_IGNORE (-Wformat-nonliteral) + GCC46_DIAG_RESTORE
    GCC30_DIAG_IGNORE (-Wformat-nonliteral) w/o
    GCC31_DIAG_IGNORE (-Wdeprecated-declarations)
+   GCC30_DIAG_IGNORE (-Wshadow)
 */
 #if defined(__GNUC__)
-#  define CC_DIAG_PRAGMA(x) _Pragma (#  x)
-#  define CLANG_DIAG_IGNORE(w)
-#  define CLANG_DIAG_RESTORE
-#elif defined(__clang__) || defined(__clang)
-#  define CC_DIAG_PRAGMA(x) _Pragma (#  x)
+#  define _GNUC_VERSION ((__GNUC__ * 100) + __GNUC_MINOR__)
+#  define CC_DIAG_PRAGMA(x) _Pragma (#x)
+#else
+#  define _GNUC_VERSION 0
+#  define CC_DIAG_PRAGMA(x)
+#endif
+
+/*
+#define _STR(x)  #x
+#define STR(x)_STR(x)
+#pragma message("_GNUC_VERSION " STR(_GNUC_VERSION))
+*/
+
+// clang-specifics (rarely needed, as they mimic GCC dignostics closely, even down to bugs)
+#if defined(__clang__) || defined(__clang)
+#  define HAVE_CLANG
 #  define CLANG_DIAG_IGNORE(x)                                                \
     _Pragma ("clang diagnostic push")                                         \
-        CC_DIAG_PRAGMA (clang diagnostic ignored #x)
+    CC_DIAG_PRAGMA (clang diagnostic ignored #x)
 #  define CLANG_DIAG_RESTORE _Pragma ("clang diagnostic pop")
+#elif defined(__GNUC__)
+#  define CLANG_DIAG_IGNORE(w)
+#  define CLANG_DIAG_RESTORE
 #else
-#  define GCC_DIAG_IGNORE(w)
+// MSVC has the __pragma() macro instead
 #  define CLANG_DIAG_IGNORE(w)
 #  define CLANG_DIAG_RESTORE
 #endif
-/* for GCC46_DIAG_IGNORE (-Wdeprecated-declarations) inside functions */
-#if (defined(__GNUC__) && ((__GNUC__ * 100) + __GNUC_MINOR__) >= 460)         \
-    || defined(__clang__) || defined(__clang)
-#  define GCC46_DIAG_PRAGMA(x) _Pragma (#  x)
+
+/* for GCC46_DIAG_IGNORE (-Wdeprecated-declarations) or (-Wformat-nonliteral),
+   stacked inside functions.
+   clang 10.2 defines gcc compat version 4.2 though (402) */
+#if _GNUC_VERSION >= 460 || (defined(HAVE_CLANG) && _GNUC_VERSION >= 400)
+#  define HAVE_CC_DIAG_STACK
 #  define GCC46_DIAG_IGNORE(x)                                                \
-    _Pragma ("GCC diagnostic push")                                           \
-        GCC46_DIAG_PRAGMA (GCC diagnostic ignored #x)
+     _Pragma ("GCC diagnostic push")                                          \
+     CC_DIAG_PRAGMA (GCC diagnostic ignored #x)
 #  define GCC46_DIAG_RESTORE _Pragma ("GCC diagnostic pop")
 #else
+#  undef HAVE_CC_DIAG_STACK
 #  define GCC46_DIAG_IGNORE(w)
 #  define GCC46_DIAG_RESTORE
 #endif
+
+/* For GCC30_DIAG_IGNORE (-Wformat-nonliteral) outside functions */
+#if _GNUC_VERSION >= 300 && !defined HAVE_CC_DIAG_STACK
+#  define GCC30_DIAG_IGNORE(x) CC_DIAG_PRAGMA (GCC diagnostic ignored #x)
+#else
+#  define GCC30_DIAG_IGNORE(w)
+#endif
 /* for GCC31_DIAG_IGNORE (-Wdeprecated-declarations) outside functions */
-#if (defined(__GNUC__) && ((__GNUC__ * 100) + __GNUC_MINOR__) >= 310          \
-     && ((__GNUC__ * 100) + __GNUC_MINOR__) < 460)
-#  define GCC31_DIAG_PRAGMA(x) _Pragma (#  x)
-#  define GCC31_DIAG_IGNORE(x) GCC31_DIAG_PRAGMA (GCC diagnostic ignored #  x)
+#if _GNUC_VERSION >= 310 && !defined HAVE_CC_DIAG_STACK
+#  define GCC31_DIAG_IGNORE(x) CC_DIAG_PRAGMA (GCC diagnostic ignored #x)
 #else
 #  define GCC31_DIAG_IGNORE(w)
 #endif
-/* For GCC30_DIAG_IGNORE (-Wformat-nonliteral) outside functions */
-#if (defined(__GNUC__) && ((__GNUC__ * 100) + __GNUC_MINOR__) >= 300          \
-     && ((__GNUC__ * 100) + __GNUC_MINOR__) < 460)
-#  define GCC30_DIAG_PRAGMA(x) _Pragma (#  x)
-#  define GCC30_DIAG_IGNORE(x) GCC30_DIAG_PRAGMA (GCC diagnostic ignored #  x)
+/* for GCC33_DIAG_IGNORE (-Wswitch-enum) outside functions
+   -Wswitch-enum appeared first with gcc 3.3.6
+ */
+#if _GNUC_VERSION >= 330 && !defined HAVE_CC_DIAG_STACK
+#  define GCC33_DIAG_IGNORE(x) CC_DIAG_PRAGMA (GCC diagnostic ignored #x)
 #else
-#  define GCC30_DIAG_IGNORE(w)
+#  define GCC33_DIAG_IGNORE(w)
 #endif
 
 #ifndef __has_feature
@@ -103,13 +127,15 @@
 
 /* The __nonnull function attribute marks pointer arguments which
    must not be NULL.  */
-#if (defined(__GNUC__) && ((__GNUC__ * 100) + __GNUC_MINOR__) >= 303)
+#if _GNUC_VERSION >= 303
 #  undef __nonnull
 #  define __nonnull(params) __attribute__ ((__nonnull__ params))
+#  define __nonnull_all __attribute__ ((__nonnull__))
 #  define HAVE_NONNULL
 #else
-#  undef HAVE_NONNULL
 #  define __nonnull(params)
+#  define __nonnull_all
+#  undef HAVE_NONNULL
 #endif
 
 #ifdef HAVE_FUNC_ATTRIBUTE_MALLOC
@@ -138,9 +164,14 @@
 #  endif
 #endif
 
+#undef CAN_ACIS_IN_DS_DATA
+#undef CAN_ACIS_HISTORY
 #define TODO_ENCODER HANDLER (OUTPUT, "TODO: Encoder\n");
 #define TODO_DECODER HANDLER (OUTPUT, "TODO: Decoder\n");
 
+// Exporters are more common in the spec format, in_json and in_dxf are not using it.
+// So default to the encode-to format. dec_macros needs to override them.
+// See importer.h for the other way: For decode, in_json, in_dxf.
 #define VERSION(v)                                                            \
   cur_ver = v;                                                                \
   if (dat->version == v)
@@ -171,12 +202,9 @@
 
 #define SAFENAME(name) (name) ? (name) : ""
 #define SAFEDXFNAME (obj && obj->dxfname ? obj->dxfname : "")
-
-extern const char version_codes[DWG_VERSIONS][7];
-
-EXPORT Dwg_Version_Type dwg_version_as (const char *);
-const char *dwg_version_type (const Dwg_Version_Type version);
-EXPORT void dwg_errstrings (int error);
+#define ARRAY_SIZE(arr) (int)(sizeof (arr) / sizeof ((arr)[0]))
+#define MIN(X, Y) ((X) < (Y) ? (X) : (Y))
+#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))
 
 /**
  Data types (including compressed forms) used through the project
@@ -207,6 +235,8 @@ typedef enum DWG_BITS
                     “string stream” within the object data. r2007+ */
   BITS_T,       /** text, version dependent: TV or TU */
   BITS_TF,      /** fixed-length text */
+  BITS_T32,     /** String32 type */
+  BITS_TU32,    /** StringU32 type (FileDepList.features) */
   BITS_HANDLE,  /** handle reference (see the HANDLE REFERENCES section) */
   BITS_BE,      /** BitExtrusion */
   BITS_DD,      /** BitDouble With Default */
@@ -226,6 +256,8 @@ typedef enum DWG_BITS
   BITS_CRC64
 } Dwg_Bits;
 
+/* Globals inside the lib */
+extern const char version_codes[DWG_VERSIONS][7];
 extern const char *dwg_bits_name[];
 extern const unsigned char dwg_bits_size[];
 
@@ -251,15 +283,57 @@ char *strrplc (const char *s, const char *from, const char *to);
 #define strEQ(s1, s2) !strcmp ((s1), (s2))
 #define strNE(s1, s2) strcmp ((s1), (s2))
 #define strEQc(s1, s2) !strcmp ((s1), s2 "")
+#define strNEc(s1, s2) strcmp ((s1), s2 "")
 
 #define memBEGIN(s1, s2, len) (strlen (s1) >= len && !memcmp (s1, s2, len))
 #define memBEGINc(s1, s2)                                                     \
   (strlen (s1) >= sizeof (s2 "") - 1 && !memcmp (s1, s2, sizeof (s2 "") - 1))
 
+#ifndef M_PI
+#  define M_PI 3.14159265358979323846
+#endif
 #ifndef M_PI_2
 #  define M_PI_2 1.57079632679489661923132169163975144
 #endif
 #define rad2deg(ang) (ang) * 90.0 / M_PI_2
 #define deg2rad(ang) (ang) * M_PI_2 / 90.0
 
+#if !defined(HAVE_MEMMEM) || defined(COMMON_TEST_C)
+// only if _GNU_SOURCE
+void *my_memmem (const void *h0, size_t k, const void *n0, size_t l) __nonnull((1, 3));
+#define memmem my_memmem
+#elif !defined(_GNU_SOURCE) && defined(IS_DECODER)
+/* HAVE_MEMMEM and _GNU_SOURCE are unreliable on non-Linux systems.
+   This fails on FreeBSD and macos.
+   Rather declare it by ourselves, and don't use _GNU_SOURCE. */
+void *memmem (const void *h0, size_t k, const void *n0, size_t l) __nonnull((1, 3));
+#endif
+
+// push to handle vector at the end. It really is unshift.
+#define PUSH_HV(_obj, numfield, hvfield, ref)                                 \
+  {                                                                           \
+    _obj->hvfield = (BITCODE_H *)realloc (                                    \
+        _obj->hvfield, (_obj->numfield + 1) * sizeof (BITCODE_H));            \
+    _obj->hvfield[_obj->numfield] = ref;                                      \
+    LOG_TRACE ("%s[%d] = " FORMAT_REF " [H]\n", #hvfield, _obj->numfield,     \
+               ARGS_REF (_obj->hvfield[_obj->numfield]));                     \
+    _obj->numfield++;                                                         \
+  }
+
+// no need to free global handles, just the HV.
+// returns the last
+#define POP_HV(_obj, numfield, hvfield) _obj->hvfield[--_obj->numfield]
+// returns the first
+#define SHIFT_HV(_obj, numfield, hvfield) shift_hv (_obj->hvfield, &_obj->numfield)
+BITCODE_H shift_hv (BITCODE_H *hv, BITCODE_BL *num_p);
+
+// used in dwg.spec
+Dwg_Handle *dwg_find_first_type_handle (Dwg_Data *restrict dwg,
+                                        enum DWG_OBJECT_TYPE type);
+
+// <path-to>/dxf.ext => copy of "dxf", "ext"
+// Returns a malloc'ed copy of basename, and
+// sets ext to the char behind the last "." of filepath
+char *split_filepath (const char *filepath, char **extp);
+
 #endif
