@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018 Free Software Foundation, Inc.                        */
+/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -21,7 +21,13 @@
 #include "dwg.h"
 #include "bits.h"
 
-EXPORT int dwg_write_json(Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
-EXPORT int dwg_write_geojson(Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+#ifdef IN_DXF_H
+#error in_dxf.h must be included after out_json.h because of FORMAT_BD
+#endif
+
+EXPORT int dwg_write_json (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+EXPORT int dwg_write_geojson (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+
+char *json_cquote (char *restrict dest, const char *restrict src, const int len);
 
 #endif
